@@ -2,7 +2,7 @@
 from dataclasses import dataclass, field
 
 import enum
-from typing import AbstractSet, Mapping, Optional, Sequence, Union
+from typing import AbstractSet, Mapping, Optional, Sequence, Type, Union
 
 from .expr import (
     BaseExpr,
@@ -16,11 +16,13 @@ from .expr import (
     OptRemotePathExpr,
     OptStrExpr,
     OptTimeDeltaExpr,
+    PrimitiveExpr,
     RemotePathExpr,
     SequenceT,
     SimpleIdExpr,
     SimpleOptBoolExpr,
     SimpleOptIdExpr,
+    SimpleOptPrimitiveExpr,
     SimpleOptStrExpr,
     SimpleStrExpr,
     StrExpr,
@@ -64,6 +66,9 @@ class Project(Base):
     images: Optional[Mapping[str, "Image"]] = field(metadata={"allow_none": True})
     volumes: Optional[Mapping[str, "Volume"]] = field(metadata={"allow_none": True})
     defaults: Optional["BatchFlowDefaults"] = field(metadata={"allow_none": True})
+    mixins: Optional[Mapping[str, "ExecUnitMixin"]] = field(
+        metadata={"allow_none": True}
+    )
 
 
 # There are 'batch' for pipelined mode and 'live' for interactive one
@@ -95,6 +100,26 @@ class Image(Base):
     force_rebuild: OptBoolExpr
 
 
+@dataclass(frozen=True)
+class ExecUnitMixin(WithSpecifiedFields, Base):
+    title: OptStrExpr  # Autocalculated if not passed explicitly
+    name: OptStrExpr
+    image: OptStrExpr
+    preset: OptStrExpr
+    schedule_timeout: OptTimeDeltaExpr
+    entrypoint: OptStrExpr
+    cmd: OptStrExpr
+    workdir: OptRemotePathExpr
+    env: Optional[BaseExpr[MappingT]] = field(metadata={"allow_none": True})
+    volumes: Optional[BaseExpr[SequenceT]] = field(metadata={"allow_none": True})
+    tags: Optional[BaseExpr[SequenceT]] = field(metadata={"allow_none": True})
+    life_span: OptTimeDeltaExpr
+    http_port: OptIntExpr
+    http_auth: OptBoolExpr
+    pass_config: OptBoolExpr
+    mixins: Optional[Sequence[StrExpr]] = field(metadata={"allow_none": True})
+
+
 @dataclass(frozen=True)
 class ExecUnit(Base):
     title: OptStrExpr  # Autocalculated if not passed explicitly
@@ -118,9 +143,9 @@ class ExecUnit(Base):
 class Matrix(Base):
     # AST class is slightly different from YAML representation,
     # in YAML `products` mapping is embedded into the matrix itself.
-    products: Mapping[str, Sequence[StrExpr]]
-    exclude: Sequence[Mapping[str, StrExpr]]
-    include: Sequence[Mapping[str, StrExpr]]
+    products: Mapping[str, BaseExpr[SequenceT]]
+    exclude: Sequence[Mapping[str, PrimitiveExpr]]
+    include: Sequence[Mapping[str, PrimitiveExpr]]
 
 
 @dataclass(frozen=True)
@@ -170,7 +195,7 @@ class JobMixin(WithSpecifiedFields, Base):
     port_forward: Optional[BaseExpr[SequenceT]] = field(metadata={"allow_none": True})
     multi: SimpleOptBoolExpr
     params: Optional[Mapping[str, Param]] = field(metadata={"allow_none": True})
-    inherits: Optional[Sequence[StrExpr]] = field(metadata={"allow_none": True})
+    mixins: Optional[Sequence[StrExpr]] = field(metadata={"allow_none": True})
 
 
 @dataclass(frozen=True)
@@ -181,7 +206,7 @@ class Job(ExecUnit, WithSpecifiedFields, JobBase):
     browse: OptBoolExpr
     port_forward: Optional[BaseExpr[SequenceT]] = field(metadata={"allow_none": True})
     multi: SimpleOptBoolExpr
-    inherits: Optional[Sequence[StrExpr]] = field(metadata={"allow_none": True})
+    mixins: Optional[Sequence[StrExpr]] = field(metadata={"allow_none": True})
 
 
 class NeedsLevel(enum.Enum):
@@ -209,7 +234,7 @@ class TaskBase(Base):
 
 @dataclass(frozen=True)
 class Task(ExecUnit, WithSpecifiedFields, TaskBase):
-    inherits: Optional[Sequence[StrExpr]] = field(metadata={"allow_none": True})
+    mixins: Optional[Sequence[StrExpr]] = field(metadata={"allow_none": True})
 
 
 @dataclass(frozen=True)
@@ -233,19 +258,19 @@ class TaskMixin(WithSpecifiedFields, Base):
     strategy: Optional[Strategy] = field(metadata={"allow_none": True})
     enable: EnableExpr = field(metadata={"default_expr": "${{ success() }}"})
     cache: Optional[Cache] = field(metadata={"allow_none": True})
-    inherits: Optional[Sequence[StrExpr]] = field(metadata={"allow_none": True})
+    mixins: Optional[Sequence[StrExpr]] = field(metadata={"allow_none": True})
 
 
 @dataclass(frozen=True)
 class BaseActionCall(Base):
     action: SimpleStrExpr  # action ref
-    args: Optional[Mapping[str, StrExpr]] = field(metadata={"allow_none": True})
+    args: Optional[Mapping[str, PrimitiveExpr]] = field(metadata={"allow_none": True})
 
 
 @dataclass(frozen=True)
 class BaseModuleCall(Base):
     module: SimpleStrExpr  # action ref
-    args: Optional[Mapping[str, StrExpr]] = field(metadata={"allow_none": True})
+    args: Optional[Mapping[str, PrimitiveExpr]] = field(metadata={"allow_none": True})
 
 
 @dataclass(frozen=True)
@@ -329,10 +354,29 @@ class ActionKind(enum.Enum):
     LOCAL = "local"  # runs locally, can be used in batch flow
 
 
+class InputType(enum.Enum):
+    INT = "int"
+    FLOAT = "float"
+    BOOL = "bool"
+    STR = "str"
+
+    def to_type(self) -> Union[Type[str], Type[float], Type[int], Type[bool]]:
+        if self.value == "int":
+            return int
+        elif self.value == "float":
+            return float
+        elif self.value == "bool":
+            return bool
+        elif self.value == "str":
+            return str
+        assert False, "Not reachable"
+
+
 @dataclass(frozen=True)
 class Input(Base):
     descr: SimpleOptStrExpr
-    default: SimpleOptStrExpr
+    default: SimpleOptPrimitiveExpr
+    type: InputType = InputType.STR
 
 
 @dataclass(frozen=True)
