@@ -1,22 +1,22 @@
 import click
 import neuro_sdk
-import sys
-from typing import List, Optional, Tuple
+from contextlib import AsyncExitStack
+from typing import List, Optional, Sequence, Tuple
 
-from neuro_flow.cli.click_types import LIVE_JOB, LIVE_JOB_OR_ALL, SUFFIX_AFTER_LIVE_JOB
+from neuro_flow.cli.click_types import (
+    LIVE_JOB,
+    LIVE_JOB_OR_ALL,
+    PROJECT,
+    LiveJobSuffixType,
+)
 from neuro_flow.cli.utils import argument, option, wrap_async
 from neuro_flow.live_runner import LiveRunner
-from neuro_flow.storage import APIStorage, NeuroStorageFS, Storage
 
+from ..storage.api import ApiStorage
+from ..storage.base import Storage
 from .root import Root
 
 
-if sys.version_info >= (3, 7):
-    from contextlib import AsyncExitStack
-else:
-    from async_exit_stack import AsyncExitStack
-
-
 @click.command()
 @wrap_async()
 async def ps(
@@ -25,9 +25,7 @@ async def ps(
     """List all jobs"""
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
-        storage: Storage = await stack.enter_async_context(
-            APIStorage(client, NeuroStorageFS(client))
-        )
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
         runner = await stack.enter_async_context(
             LiveRunner(root.config_dir, root.console, client, storage, root)
         )
@@ -37,7 +35,11 @@ async def ps(
 @click.command()
 @option("-s", "--suffix", help="Optional suffix for multi-jobs")
 @option(
-    "--param", type=(str, str), multiple=True, help="Set params of the batch config"
+    "-p",
+    "--param",
+    type=(str, str),
+    multiple=True,
+    help="Set params of the batch config",
 )
 @option(
     "--dry-run",
@@ -68,24 +70,21 @@ async def run(
         )
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
-        storage: Storage = await stack.enter_async_context(
-            APIStorage(client, NeuroStorageFS(client))
-        )
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
         runner = await stack.enter_async_context(
-            LiveRunner(root.config_dir, root.console, client, storage, root)
+            LiveRunner(root.config_dir, root.console, client, storage, root, dry_run)
         )
         await runner.run(
             job_id,
             suffix=suffix,
             args=args,
             params={key: value for key, value in param},
-            dry_run=dry_run,
         )
 
 
 @click.command()
 @argument("job-id", type=LIVE_JOB)
-@argument("suffix", required=False, type=SUFFIX_AFTER_LIVE_JOB)
+@argument("suffix", required=False, type=LiveJobSuffixType(job_id_param_name="job_id"))
 @wrap_async()
 async def logs(
     root: Root,
@@ -98,9 +97,7 @@ async def logs(
     """
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
-        storage: Storage = await stack.enter_async_context(
-            APIStorage(client, NeuroStorageFS(client))
-        )
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
         runner = await stack.enter_async_context(
             LiveRunner(root.config_dir, root.console, client, storage, root)
         )
@@ -109,7 +106,7 @@ async def logs(
 
 @click.command()
 @argument("job-id", type=LIVE_JOB)
-@argument("suffix", required=False, type=SUFFIX_AFTER_LIVE_JOB)
+@argument("suffix", required=False, type=LiveJobSuffixType(job_id_param_name="job_id"))
 @wrap_async()
 async def status(
     root: Root,
@@ -122,9 +119,7 @@ async def status(
     """
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
-        storage: Storage = await stack.enter_async_context(
-            APIStorage(client, NeuroStorageFS(client))
-        )
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
         runner = await stack.enter_async_context(
             LiveRunner(root.config_dir, root.console, client, storage, root)
         )
@@ -133,7 +128,7 @@ async def status(
 
 @click.command()
 @argument("job-id", type=LIVE_JOB_OR_ALL)
-@argument("suffix", required=False, type=SUFFIX_AFTER_LIVE_JOB)
+@argument("suffix", required=False, type=LiveJobSuffixType(job_id_param_name="job_id"))
 @wrap_async()
 async def kill(
     root: Root,
@@ -145,9 +140,7 @@ async def kill(
     Kill JOB-ID, use `kill ALL` for killing all jobs."""
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
-        storage: Storage = await stack.enter_async_context(
-            APIStorage(client, NeuroStorageFS(client))
-        )
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
         runner = await stack.enter_async_context(
             LiveRunner(root.config_dir, root.console, client, storage, root)
         )
@@ -159,3 +152,20 @@ async def kill(
                     "Suffix is not supported when killing ALL jobs"
                 )
             await runner.kill_all()
+
+
+@click.command()
+@argument("project_ids", type=PROJECT, nargs=-1, required=True)
+@wrap_async()
+async def delete_project(
+    root: Root,
+    project_ids: Sequence[str],
+) -> None:
+    """Completely remove project with all related entities"""
+    async with AsyncExitStack() as stack:
+        client = await stack.enter_async_context(neuro_sdk.get())
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
+        for project_id in project_ids:
+            await storage.project(yaml_id=project_id).delete()
+            if root.verbosity >= 0:
+                root.console.print(f"Project '{project_id}' was successfully removed.")
