@@ -1,18 +1,13 @@
 import click
 import neuro_sdk
-import sys
+from contextlib import AsyncExitStack
 
 from neuro_flow.cli.click_types import LIVE_VOLUME_OR_ALL
 from neuro_flow.cli.root import Root
 from neuro_flow.cli.utils import argument, wrap_async
 from neuro_flow.live_runner import LiveRunner
-from neuro_flow.storage import APIStorage, NeuroStorageFS, Storage
-
-
-if sys.version_info >= (3, 7):
-    from contextlib import AsyncExitStack
-else:
-    from async_exit_stack import AsyncExitStack
+from neuro_flow.storage.api import ApiStorage
+from neuro_flow.storage.base import Storage
 
 
 @click.command()
@@ -28,9 +23,7 @@ async def upload(
     use `upload ALL` for uploading all volumes."""
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
-        storage: Storage = await stack.enter_async_context(
-            APIStorage(client, NeuroStorageFS(client))
-        )
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
         runner = await stack.enter_async_context(
             LiveRunner(root.config_dir, root.console, client, storage, root)
         )
@@ -53,9 +46,7 @@ async def download(
     use `download ALL` for downloading all volumes."""
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
-        storage: Storage = await stack.enter_async_context(
-            APIStorage(client, NeuroStorageFS(client))
-        )
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
         runner = await stack.enter_async_context(
             LiveRunner(root.config_dir, root.console, client, storage, root)
         )
@@ -78,9 +69,7 @@ async def clean(
     use `clean ALL` for cleaning up all volumes."""
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
-        storage: Storage = await stack.enter_async_context(
-            APIStorage(client, NeuroStorageFS(client))
-        )
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
         runner = await stack.enter_async_context(
             LiveRunner(root.config_dir, root.console, client, storage, root)
         )
@@ -98,9 +87,7 @@ async def mkvolumes(
     """Create all remote folders for volumes."""
     async with AsyncExitStack() as stack:
         client = await stack.enter_async_context(neuro_sdk.get())
-        storage: Storage = await stack.enter_async_context(
-            APIStorage(client, NeuroStorageFS(client))
-        )
+        storage: Storage = await stack.enter_async_context(ApiStorage(client))
         runner = await stack.enter_async_context(
             LiveRunner(root.config_dir, root.console, client, storage, root)
         )
