@@ -1,6 +1,8 @@
+from dataclasses import dataclass
+
 import enum
 from pathlib import Path, PurePosixPath
-from typing import ClassVar, Tuple
+from typing import ClassVar, List, Sequence, Tuple
 
 
 LocalPath = Path
@@ -35,6 +37,36 @@ class TaskStatus(str, enum.Enum):
     SKIPPED = "skipped"
     CACHED = "cached"
 
+    @property
+    def is_pending(self) -> bool:
+        return self == self.PENDING
+
+    @property
+    def is_running(self) -> bool:
+        return self == self.RUNNING
+
+    @property
+    def is_finished(self) -> bool:
+        return self in (
+            self.SUCCEEDED,
+            self.FAILED,
+            self.CANCELLED,
+            self.SKIPPED,
+            self.CACHED,
+        )
+
+    @classmethod
+    def values(cls) -> List[str]:
+        return [item.value for item in cls]
+
+    @classmethod
+    def active_values(cls) -> List[str]:
+        return [item.value for item in cls if not item.is_finished]
+
+    @classmethod
+    def finished_values(cls) -> List[str]:
+        return [item.value for item in cls if item.is_finished]
+
 
 COLORS = {
     TaskStatus.UNKNOWN: "reverse bright_black",
@@ -53,6 +85,7 @@ class ImageStatus(str, enum.Enum):
     BUILDING = "building"
     BUILT = "built"
     BUILD_FAILED = "build_failed"
+    CACHED = "cached"
 
     def __rich__(self) -> str:
         return f"[{IMAGE_STATUS_COLORS[self]}]{self}"
@@ -63,4 +96,12 @@ IMAGE_STATUS_COLORS = {
     ImageStatus.BUILDING: "blue",
     ImageStatus.BUILT: "green",
     ImageStatus.BUILD_FAILED: "red",
+    ImageStatus.CACHED: "magenta",
 }
+
+
+@dataclass(frozen=True)
+class GitInfo:
+    sha: str
+    branch: str
+    tags: Sequence[str]
