@@ -1,6 +1,7 @@
 from dataclasses import dataclass
 
 import aiohttp
+import asyncio
 import logging
 import os
 import pathlib
@@ -8,10 +9,20 @@ import pytest
 import secrets
 import shutil
 import subprocess
+import sys
 from datetime import datetime, timedelta
 from neuro_sdk import Config, get as api_get, login_with_token
 from pathlib import Path
-from typing import Any, AsyncIterator, Callable, Dict, List, Optional
+from typing import (
+    Any,
+    AsyncIterator,
+    Awaitable,
+    Callable,
+    Dict,
+    Iterator,
+    List,
+    Optional,
+)
 from yarl import URL
 
 from neuro_flow.context import sanitize_name
@@ -23,6 +34,15 @@ CLIENT_TIMEOUT = aiohttp.ClientTimeout(None, None, NETWORK_TIMEOUT, NETWORK_TIME
 log = logging.getLogger(__name__)
 
 
+@pytest.fixture
+def loop() -> Iterator[asyncio.AbstractEventLoop]:
+    loop = asyncio.new_event_loop()
+    asyncio.set_event_loop(loop)
+    yield loop
+    loop.run_until_complete(loop.shutdown_asyncgens())
+    loop.close()
+
+
 @pytest.fixture
 def assets() -> pathlib.Path:
     return pathlib.Path(__file__).parent / "assets"
@@ -30,7 +50,7 @@ def assets() -> pathlib.Path:
 
 @pytest.fixture
 def project_id() -> str:
-    return f"e2e_proj_{make_image_date_flag()}_{secrets.token_hex(10)}"
+    return f"e2e_proj_{secrets.token_hex(6)}_{make_date_flag()}"
 
 
 async def get_config(nmrc_path: Optional[Path]) -> Config:
@@ -62,7 +82,7 @@ async def project_role(
     try:
         yield project_role
     finally:
-        run_neuro_cli(["acl", "remove-role", project_role])
+        await run_neuro_cli(["acl", "remove-role", project_role])
 
 
 @pytest.fixture
@@ -78,7 +98,7 @@ def ws(
     project_data = f"id: {project_id}"
     if username:
         project_data += f'\nowner: "{username}"'
-    (ws_dir / "project.yml").write_text(project_data)
+    (ws_dir / ".neuro" / "project.yml").write_text(project_data)
     return ws_dir
 
 
@@ -104,34 +124,40 @@ class SysCap:
     err: str
 
 
-RunCLI = Callable[[List[str]], SysCap]
+RunCLI = Callable[[List[str]], Awaitable[SysCap]]
 
 
 @pytest.fixture
 def _run_cli(
     loop: None, ws: pathlib.Path, api_config: Optional[pathlib.Path]
 ) -> RunCLI:
-    def _run(
+    async def _run(
         arguments: List[str],
     ) -> SysCap:
         if api_config:
             os.environ["NEUROMATION_CONFIG"] = str(api_config)
-        proc = subprocess.run(
-            arguments,
-            timeout=600,
+        proc = await asyncio.create_subprocess_exec(
+            *arguments,
             cwd=ws,
-            encoding="utf8",
-            errors="replace",
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
         )
-        try:
-            proc.check_returncode()
-        except subprocess.CalledProcessError:
-            log.error(f"Last stdout: '{proc.stdout}'")
-            log.error(f"Last stderr: '{proc.stderr}'")
-            raise
-        return SysCap(out=proc.stdout.strip(), err=proc.stderr.strip())
+        assert proc.stdout
+        assert proc.stderr
+        ret_code = await asyncio.wait_for(proc.wait(), timeout=600)
+        out = (await proc.stdout.read()).decode(
+            encoding="utf8",
+            errors="replace",
+        )
+        err = (await proc.stderr.read()).decode(
+            encoding="utf8",
+            errors="replace",
+        )
+        if ret_code:
+            log.error(f"Last stdout: '{out}'")
+            log.error(f"Last stderr: '{err}'")
+            raise subprocess.CalledProcessError(ret_code, arguments[0], out, err)
+        return SysCap(out=out.strip(), err=err.strip())
 
     return _run
 
@@ -146,13 +172,13 @@ def run_neuro_cli(_run_cli: RunCLI) -> RunCLI:
     return lambda args: _run_cli(["neuro", "--show-traceback"] + args)
 
 
-IMAGE_DATETIME_FORMAT = "%Y%m%d%H%M"
-IMAGE_DATETIME_SEP = "_date"
+DATETIME_FORMAT = "%Y%m%d%H%M"
+DATETIME_SEP = "date"
 
 
-def make_image_date_flag() -> str:
-    time_str = datetime.now().strftime(IMAGE_DATETIME_FORMAT)
-    return f"{IMAGE_DATETIME_SEP}{time_str}{IMAGE_DATETIME_SEP}"
+def make_date_flag() -> str:
+    time_str = datetime.now().strftime(DATETIME_FORMAT)
+    return f"{DATETIME_SEP}{time_str}{DATETIME_SEP}"
 
 
 @pytest.fixture(scope="session")
@@ -161,24 +187,72 @@ def _drop_once_flag() -> Dict[str, bool]:
 
 
 @pytest.fixture(autouse=True)
-def drop_old_test_images(
+async def drop_old_test_images(
     run_neuro_cli: RunCLI, _drop_once_flag: Dict[str, bool]
 ) -> None:
-    if _drop_once_flag.get("cleaned"):
+    if _drop_once_flag.get("cleaned_images"):
         return
 
-    res: SysCap = run_neuro_cli(["-q", "image", "ls", "--full-uri"])
-    for image_str in res.out.splitlines():
+    if sys.version_info < (3, 8) and sys.platform == "win32":
+        # Default selector-based event loop for Windows doesn't support subprocesses
+        return
+
+    async def _drop_iamge(image_str: str) -> None:
         image_str = image_str.strip()
         image_url = URL(image_str)
         image_name = image_url.parts[-1]
         try:
-            _, time_str, _ = image_name.split(IMAGE_DATETIME_SEP)
-            image_time = datetime.strptime(time_str, IMAGE_DATETIME_FORMAT)
+            _, time_str, _ = image_name.split(DATETIME_SEP)
+            image_time = datetime.strptime(time_str, DATETIME_FORMAT)
             if datetime.now() - image_time < timedelta(days=1):
-                continue
-            run_neuro_cli(["image", "rm", image_str])
+                return
+            await run_neuro_cli(["image", "rm", image_str])
         except Exception:
             pass
 
-    _drop_once_flag["cleaned"] = True
+    res: SysCap = await run_neuro_cli(["-q", "image", "ls", "--full-uri"])
+
+    tasks = []
+    for image_str in res.out.splitlines():
+        tasks.append(asyncio.ensure_future(_drop_iamge(image_str)))
+
+    if tasks:
+        await asyncio.wait(tasks)
+
+    _drop_once_flag["cleaned_images"] = True
+
+
+@pytest.fixture(autouse=True)
+async def drop_old_roles(
+    run_neuro_cli: RunCLI, _drop_once_flag: Dict[str, bool], username: str
+) -> None:
+    if _drop_once_flag.get("cleaned_roles"):
+        return
+
+    res: SysCap = await run_neuro_cli(
+        ["acl", "ls", "--shared", f"role://{username}/projects"]
+    )
+
+    async def _drop_role(project_str: str) -> None:
+        role_str, _ = project_str.split(" ", 1)
+        role_uri = URL(role_str)
+        role_name = role_uri.parts[-1]
+        try:
+            _, time_str, _ = role_name.split(DATETIME_SEP)
+            proj_time = datetime.strptime(time_str, DATETIME_FORMAT)
+            if datetime.now() - proj_time < timedelta(hours=4):
+                return
+            await run_neuro_cli(
+                ["acl", "remove-role", f"{username}/projects/{role_name}"]
+            )
+        except Exception:
+            pass
+
+    tasks = []
+    for project in res.out.splitlines():
+        tasks.append(asyncio.ensure_future(_drop_role(project)))
+
+    if tasks:
+        await asyncio.wait(tasks)
+
+    _drop_once_flag["cleaned_roles"] = True
