@@ -1,7 +1,7 @@
 import pytest
-import sys
+from contextlib import asynccontextmanager
 from neuro_sdk import Client
-from typing import AsyncIterator, Dict, List
+from typing import Any, AsyncIterator, Dict, List
 from typing_extensions import Final
 
 from neuro_flow.context import DepCtx
@@ -19,20 +19,17 @@ from neuro_flow.tokenizer import Pos, tokenize
 from neuro_flow.types import LocalPath, TaskStatus
 
 
-if sys.version_info >= (3, 7):  # pragma: no cover
-    from contextlib import asynccontextmanager
-else:
-    from async_generator import asynccontextmanager
-
-
 FNAME = LocalPath("<test>")
 START: Final = Pos(0, 0, FNAME)
 
 
 class DictContext(RootABC):
-    def __init__(self, dct: Dict[str, TypeT], client: Client) -> None:
+    def __init__(
+        self, dct: Dict[str, TypeT], client: Client, dry_run: bool = False
+    ) -> None:
         self._dct = dct
         self._client = client
+        self._dry_run = dry_run
 
     def lookup(self, name: str) -> TypeT:
         return self._dct[name]
@@ -41,6 +38,10 @@ class DictContext(RootABC):
     async def client(self) -> AsyncIterator[Client]:
         yield self._client
 
+    @property
+    def dry_run(self) -> bool:
+        return self._dry_run
+
 
 @pytest.mark.parametrize(
     "expr,context,result",
@@ -232,3 +233,41 @@ async def test_dict_trailing_comma(client: Client) -> None:
     pat = "${{ {'a': 1, 'b': 2,} }}"
     expr = MappingExpr(START, Pos(0, len(pat), FNAME), pat, int)  # type: ignore
     assert {"a": 1, "b": 2} == await expr.eval(DictContext({}, client))  # type: ignore
+
+
+@pytest.mark.parametrize(
+    "expr,context,result",
+    [
+        ("2 if True else 3", {}, 2),
+        ("2 if False else 3", {}, 3),
+        ("2 if True else 0 / 0", {}, 2),
+        ("0 / 0 if False else 2", {}, 2),
+        ("0 / 0 if False else 2", {}, 2),
+        (
+            "2 if some_ctx else 3",
+            {
+                "some_ctx": True,
+            },
+            2,
+        ),
+        ("2 if (3 if (4 if False else False) else False) else 3", {}, 3),
+    ],
+)
+async def test_if_else_expr(
+    client: Client, expr: str, context: Dict[str, Any], result: Any
+) -> None:
+    parsed = PARSER.parse(list(tokenize("${{" + expr + "}}", START)))
+    assert len(parsed) == 1
+    assert result == await parsed[0].eval(DictContext(context, client))
+
+
+async def test_list_comp(client: Client) -> None:
+    pat = "${{ [x * x for x in range(5)] }}"
+    expr = SequenceExpr(START, Pos(0, len(pat), FNAME), pat, int)  # type: ignore
+    assert [0, 1, 4, 9, 16] == await expr.eval(DictContext({}, client))  # type: ignore
+
+
+async def test_list_comp_with_if(client: Client) -> None:
+    pat = "${{ [x * x for x in range(5) if x % 2 == 0] }}"
+    expr = SequenceExpr(START, Pos(0, len(pat), FNAME), pat, int)  # type: ignore
+    assert [0, 4, 16] == await expr.eval(DictContext({}, client))  # type: ignore
